function [loglikelihood,ptseries,ntseries,likelihoods] = bege_gjrgarch_likelihood(data, params)
% Computing the likelihood of the time series under BEGE-GJR-GARCH dynamics, given observed data and model parameters
%
% Input:
% data - t by 1 vector containing the time series of observations to be evaluated under BEGE-GJR-GARCH dynamics
% params - the vector containing the BEGE-GJR-GARCH parameters in the
% following order (see Bekaert, G., Engstrom, E., and A. Ermolov, 2015, Bad environments, good
% environments: A non-Gaussian asymmetric volatility model, Journal of Econometrics, 1 (186), pp. 258275, for the notation):
%
% params(1)=mu;
% params(2)=p_bar;
% params(3)=sigma_p;
% params(4)=rho_p;
% params(5)=n_bar;
% params(6)=sigma_n;
% params(7)=rho_n;
% params(8)=phi_p^+;
% params(9)=phi_p^-;
% params(10)=phi_n^+;
% params(11)=phi_n^-;
%
% Output:
% loglikelihood - loglikelihood of the time series
% pt_series - time series of p_t state variables
% nt_series - time series of n_t state variables
% likelihoods - time series of individual observations likelihoods
%
% Example: 
% stockreturns=randn(250,1);%Generating data
% params=[0 1 1.5 0.9 0.3 2.5 0.8 0.2 0.1 0.1 0.2]; %Setting up the parameters
% [loglikelihood,ptseries,ntseries,likelihoods] = bege_gjrgarch_likelihood(stockreturns, params);
%
% If you are using this code, please cite the following paper:
%
% Bekaert, G., Engstrom, E., and A. Ermolov, 2015, Bad environments, good
% environments: A non-Gaussian asymmetric volatility model, Journal of Econometrics, 1 (186), pp. 258275. 
%
% The code is absolutely free to use, but we are not responsible for any
% damage it may create.

%%%%%%%%%%%%%%%%%%%
%SETTING PARAMETERS
%%%%%%%%%%%%%%%%%%%

r_bar=params(1);
p_bar=params(2);
tp=params(3);
rho_p=params(4);
phi_pp=params(8);
phi_pn=params(9);
n_bar=params(5);
tn=params(6);
rho_n=params(7);
phi_np=params(10);
phi_nn=params(11);

%Vector containing likelihood for every observation
likelihoods=zeros(length(data),1);

%Vector containing likelihood for every observation
likelihoods=zeros(length(data),1);

%Computing underlying pt and nt processes
ptseries=zeros(length(data),1);
ntseries=zeros(length(data),1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%COMPUTING THE LOG-LIKELIHOOD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

loglikelihood=0;
t1=10e-5;

previous_p=max(p_bar/(1-rho_p-(phi_pp+phi_pn)/2),t1);
previous_n=max(n_bar/(1-rho_n-(phi_np+phi_nn)/2),t1);
ptseries(1)=max(previous_p,t1);
ntseries(1)=max(previous_n,t1);

loglikelihood=loglikelihood+loglikedgam(data(1)-r_bar,ptseries(1),ntseries(1),tp,tn,0.001);
likelihoods(1)=loglikedgam(data(1)-r_bar,ptseries(1),ntseries(1),tp,tn,0.001);

for t=2:length(data)
    if ((data(t-1)-r_bar)<0)
       p_t=max(p_bar+rho_p*previous_p+...
           phi_pn*(((data(t-1)-r_bar)^2)/(2*(tp^2))),t1);
       n_t=max(n_bar+rho_n*previous_n+...
           phi_nn*(((data(t-1)-r_bar)^2)/(2*(tn^2))),t1);
    else
       p_t=max(p_bar+rho_p*previous_p+...
           phi_pp*(((data(t-1)-r_bar)^2)/(2*(tp^2))),t1);
       n_t=max(n_bar+rho_n*previous_n+...
           phi_np*(((data(t-1)-r_bar)^2)/(2*(tn^2))),t1);
    end
    obs=data(t)-r_bar;
    loglikelihood=loglikelihood+loglikedgam(obs,p_t,n_t,tp,tn,0.001);
    likelihoods(t)=loglikedgam(obs,p_t,n_t,tp,tn,0.001);
    ptseries(t)=p_t;
    ntseries(t)=n_t;
    previous_p=p_t;
    previous_n=n_t;
end

